#include "comunicacion_linux.h"
#include "errores.h"
//#include "dbeventos.h"
#include <ace/INET_Addr.h>
#include <ace/Synch_Options.h>
#include <ace/Log_Msg.h>
#include "ace/os_include/netinet/os_tcp.h"
#include <libconf/libconf.h>
#if defined(__linux__)
#  include <endian.h>
#elif defined(__FreeBSD__) || defined(__NetBSD__)
#  include <sys/endian.h>
#elif defined(__OpenBSD__)
#  include <sys/types.h>
#  define be16toh(x) betoh16(x)
#  define be32toh(x) betoh32(x)
#  define be64toh(x) betoh64(x)
#endif

using namespace picvisa;
static bool alarma_conectada = false;
//dbeventos alarma;
static char *nombre_bd, *nombre_usuario, *pwd_usuario, *nombre_role;
static void* configuracion_dring;
static char seccion[32];

//tdb dbevents;

const unsigned char orden_byte[] = {0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80};
comando_drv run = {'r'};
comando_drv paro = {'s'};
comunicacion_excepcion_error::comunicacion_excepcion_error(const std::string& /*e*/, int /*cod_tipo*/, int /*cod_error*/, int /*cod_ayuda*/, int /*id_modulo*/)
{
}

const char* comunicacion_excepcion_error::what()const throw()
{
	return err_msg.c_str();
}

comunicacion_linux::comunicacion_linux(void* conf_dring, int _modulo_id)
{
	this->modulo_id = _modulo_id;
	configuracion_dring = conf_dring;
	sprintf(seccion, "EXPULSION%i", modulo_id);
	if(obtener_entero(configuracion_dring, seccion, "puerto_linux", &puerto_linux)<0) {
		ajustar_entero(configuracion_dring, seccion, "puerto_linux", DEF_PUERTO_LINUX);
		puerto_linux = DEF_PUERTO_LINUX;
	}

	char ip[256];
	if(obtener_cadena(configuracion_dring, seccion, "nombre_fichero_driver", ip, 255)<0) {
		ajustar_cadena(configuracion_dring, seccion, "nombre_fichero_driver", DEF_NOMBRE_FICHERO_DRIVER, strlen(DEF_NOMBRE_FICHERO_DRIVER));
		strcpy(ip, DEF_NOMBRE_FICHERO_DRIVER);
	}
	nombre_fichero=ip;
	if(obtener_cadena(configuracion_dring, seccion, "ip_linux", ip, 15)<0) {
		ajustar_cadena(configuracion_dring, seccion, "ip_linux", DEF_IP_LINUX, 15);
		strcpy(ip, DEF_IP_LINUX);
	}
//	ACE_DEBUG((LM_DEBUG, "Configuracion [%s]: ip[%s] puerto[%i]\n", fichero, ip, puerto_linux));

	if(strlen(ip) > 0)
	{
		ip_linux = (char*)ACE_OS_Memory::malloc(strlen(ip) + 1);
		//ip_linux = new char[strlen(ip) + 1];
		strcpy(ip_linux, ip);
	}
	else
		ip_linux = NULL;
	
	datos.comando = ORDEN_LINEAS;
    comando_run.comando = ORDEN_RUN;
    comando_stop.comando = ORDEN_STOP;
	config_lineas.comando = ORDEN_CONFIGURACION;	
    int tmp;
	if(obtener_entero(configuracion_dring, seccion, "numero_filas", (int*)&tmp)<0) {
		ajustar_entero(configuracion_dring, seccion, "numero_filas", DEF_NUMERO_FILAS);
		 tmp = DEF_NUMERO_FILAS;
	}
    config_lineas.numero_filas = tmp;
    config_lineas.divisor = 1;
    config_lineas.reserva1 = config_lineas.reserva2 = config_lineas.reserva3 = config_lineas.reserva4 = 0;
    obtener_parametros_GUI();
}

void comunicacion_linux::run() {
    ssize_t sent_bytes = socket_aceptado.send_n(&comando_run, sizeof(comando_run));
    if(sent_bytes != sizeof(comando_run)) {
        ACE_DEBUG((LM_DEBUG, "%T Error al enviar: solo %i bytes de %i\n", sent_bytes, sizeof(comando_run)));
    }
}



void comunicacion_linux::stop() {
    ssize_t sent_bytes = socket_aceptado.send_n(&comando_stop, sizeof(comando_stop));
    if(sent_bytes != sizeof(comando_stop)) {
        ACE_DEBUG((LM_DEBUG, "%T Error al enviar: solo %i bytes de %i\n", sent_bytes, sizeof(comando_run)));
    }
}

void comunicacion_linux::configura() {
    ACE_DEBUG((LM_INFO, "Configuracion del driver: retardo %i lineas %i max_retrasos %i divisor %i\n ", config_lineas.retardo, config_lineas.numero_filas, config_lineas.max_retrasos, config_lineas.divisor));
    configuracion_lineas config_network_format;
    config_network_format.comando = config_lineas.comando;
    config_network_format.padding = 0;
    config_network_format.retardo = htons(config_lineas.retardo);
    config_network_format.numero_filas = htons(config_lineas.numero_filas);
    config_network_format.max_retrasos = htons(config_lineas.max_retrasos);
    config_network_format.divisor = htons(config_lineas.divisor);
    config_network_format.reserva1 = htons(config_lineas.reserva1);
    config_network_format.reserva2 = htons(config_lineas.reserva2);
    config_network_format.reserva3 = htons(config_lineas.reserva3);
    config_network_format.reserva4 = htons(config_lineas.reserva4);
    ssize_t sent_bytes = socket_aceptado.send_n(&config_network_format, sizeof(config_network_format));
    if(sent_bytes != sizeof(config_lineas)) {
        ACE_DEBUG((LM_DEBUG, "%T Error al enviar: solo %i bytes de %i\n", sent_bytes, sizeof(comando_run)));        
    }
}

comunicacion_linux::~comunicacion_linux()
{
	desconectar();
	if(ip_linux)
		ACE_OS_Memory::free(ip_linux);
}


void comunicacion_linux::tratar_matriz(matriz_expulsion* m_expulsion)
{
	static int fila = 0;
	ACE_Time_Value timeout (0,0);
    iovec buffs[3];
	timeout.msec(2);
    datos.tiempo = turno+config_lineas.retardo;
	for(unsigned int jj = 0; jj < config_lineas.numero_filas; jj++)
	{
		++turno;
		for(unsigned int ii = 0; ii < m_expulsion->numero_columnas; ii++){
			if(m_expulsion->inicio_matriz[ii + (jj * m_expulsion->numero_columnas)])
				datos.lineas[jj].datos[ii/8] |= orden_byte[(ii%8)]; // Activa el bit correspondiente, tanto si ya est activado como sino.
			else
				datos.lineas[jj].datos[ii/8] &= (unsigned char)(255 - orden_byte[(ii%8)]); // Desactiva el bit correspondiente, tanto si ya est activado como sino.
		}
	}
	size_t sent_bytes = 0, bytes = sizeof(datos.comando)+sizeof(datos.tiempo)+sizeof(linea_salida)*config_lineas.numero_filas;
    buffs[0].iov_len=sizeof(datos.comando);
    buffs[0].iov_base=&datos.comando;
	buffs[1].iov_len=sizeof(datos.tiempo);
	buffs[1].iov_base=&datos.tiempo;
	buffs[2].iov_len= sizeof(linea_salida)*config_lineas.numero_filas;
	buffs[2].iov_base=&datos.lineas;
	socket_aceptado.sendv_n(buffs, 3, &timeout, &sent_bytes);
/*        ACE_DEBUG((LM_DEBUG, "LINEAS ENVIADAS %d: %d bytes de %d\n", turno, bytes, sent_bytes));*/
	if(bytes!=sent_bytes) {
		ACE_DEBUG((LM_DEBUG, "%T MODEXPULSION2 ERROR  EN LINEAS ENVIADAS %l: %i bytes de %i\n", turno, bytes, sent_bytes));
	}
	timeout.set(0);
	timeout.msec(2);
	datos_driver estado_driver;
	bytes=socket_aceptado.recv_n(&estado_driver, sizeof(estado_driver), &timeout);
	if(bytes!=sizeof(estado_driver)) {
		ACE_DEBUG((LM_DEBUG, "%T MODEXPULSION2 Error en recepcion de retrasos %i\n", bytes));
	}

        
	while(bytes==sizeof(estado_driver)){
		timeout.set(0);
		timeout.usec(100);
		bytes=socket_aceptado.recv_n(&estado_driver, sizeof(estado_driver), &timeout);
	}
        
	if(turno%50==0) {
		ACE_DEBUG((LM_DEBUG, "%T Estado del driver\n\tturno: %d\n\tretraso total: %d\n\tretraso parcial: %d\n\tdesbordes: %d\n",
				   estado_driver.turno, estado_driver.retrasos_total, estado_driver.retrasos_parcial, estado_driver.desbordes));
	}
}


bool comunicacion_linux::conectar()
{
	//Declaramos el timeout para establecer la conexin.
//	ACE_Time_Value timeout (100);
    ACE_DEBUG((LM_INFO, "Conectando a (ip %s, puerto %i).\n", ip_linux, puerto_linux));
	ACE_INET_Addr direccion(puerto_linux, ip_linux);
	//ACE_FILE_Connector connector(socket_aceptado,	ACE_FILE_Addr(nombre_fichero.c_str()) );
//	conexion.connect(socket_aceptado, direccion, 0, ACE_Addr::sap_any, 0, O_RDWR);
	//fichero_dev = open(nombre_fichero.c_str(), O_WRONLY);
	ACE_DEBUG((LM_DEBUG, "Nombre fichero %s\n", ip_linux));
	//ACE_DEBUG((LM_DEBUG, "Nombre fichero %s\n", nombre_fichero.c_str()));
	// Debemos hacer una conexin <blocking>
	
	if(conexion.connect(socket_aceptado, direccion) == -1)
	{
		char mensaje[1000];
		*mensaje = '\0';
		if(errno == ETIME) // Timeout agotado debemos lanzar una excepcin
		    ACE_DEBUG((LM_INFO, "Error  tiempo expirado al intentar conectar con el controlador de expulsion (ip %s, puerto %i).\n", ip_linux, puerto_linux));
		else //Error no catalogado.
		    ACE_DEBUG((LM_INFO, "Error desconocido al conecta de expulsion (ip %s, puerto %i errno %i).\n", ip_linux, puerto_linux, ACE::sock_error(errno)));
                //	throw comunicacion_excepcion_error(mensaje, ALARMAS_TIPO_EVENTO, ALARMAS_ERR_CONEXION, ALARMAS_AYUDA_SOCKET, modulo_id);
	}
	int opt=1;
	socket_aceptado.set_option(IPPROTO_TCP, TCP_NODELAY, (char *)&opt, sizeof(opt));
	turno = 0;
	bzero(&datos, sizeof(datos_lineas));
	datos.comando = ORDEN_LINEAS;
	return true;
}

void comunicacion_linux::desconectar()
{
	//ACE_DEBUG((LM_DEBUG, "Cerrando fichero...\n"));
	socket_aceptado.close();
	//close(fichero_dev);
}

void comunicacion_linux::getset_entero(char* key, int* val, int def)
{
    if(obtener_entero(configuracion_dring, seccion, key, val)<0) {
        ajustar_entero(configuracion_dring, seccion, key, def);
        *val =def;
    }
}

void comunicacion_linux::obtener_parametros_GUI()
{
    int retardo, numero_salidas_driver1, numero_salidas_driver2;
    getset_entero("retardo", (int*)&retardo, DEF_RETARDO);
    getset_entero("numero_salidas_driver1", (int*)&numero_salidas_driver1,0);
    getset_entero("numero_salidas_driver2", (int*)&numero_salidas_driver2,8);
    getset_entero("max_retrasos", (int*)&max_retrasos,100);

    config_lineas.retardo = retardo;
    config_lineas.max_retrasos = max_retrasos;
    config_lineas.reserva1 = numero_salidas_driver1;
    config_lineas.reserva2 = numero_salidas_driver2;

}
