 /**
 * \file comunicacion_linux.h
 * \brief Contiene la clase que permite comunicar-se con el ordenador encargado de la expulsin.
 * 
 *
*/
#ifndef PV_COMUNICACION_LINUX_H
#define PV_COMUNICACION_LINUX_H

#include <iostream>
#include <string.h>
#include <ace/SOCK_Connector.h>
#include <ace/SOCK_Stream.h>
#include <ace/SOCK_Connector.h>
#include <ace/SOCK_Stream.h>
#include <ace/FILE_Connector.h>
#include "matriz_expulsion.h"
#include "str_lineas.h"

namespace picvisa
{

// Valores por defecto de los parametros de configuracion
#define DEF_PUERTO_LINUX 5505
#define DEF_IP_LINUX ""
#define DEF_NUMERO_SALIDAS_DRIVER1 12
#define DEF_NUMERO_SALIDAS_DRIVER2 12
#define DEF_NUMERO_FILAS 256
#define DEF_RETARDO 0
#define DEF_DBNAME ""
#define DEF_UID ""
#define DEF_PWD ""
#define DEF_ROLE ""
#define DEF_NOMBRE_FICHERO_DRIVER "/dev/ecog0"

/*! \struct tdb 
 * 	\brief Estructura de parametros para configurar la base de datos
 */
struct tdb {
	char dbname[256];
	char uid[32];
	char pwd[32];
	char role[32];
};

/** \class comunicacion_excepcion_error
 * \brief Clase que constituye las excepciones que se produzcan en la clase comunicacion_linux.
 */
class comunicacion_excepcion_error: public std::exception
{
	public:
		comunicacion_excepcion_error(const std::string& e, int cod_tipo, int cod_error, int cod_ayuda, int id_modulo);
		~comunicacion_excepcion_error()throw(){};
		virtual const char*what()const throw();
	private:
		std::string err_msg;
};

/** \class comunicacion_linux
 * \brief Clase que se encarga de la comunicacin con el ordenador servidor
 * de la expulsin.
 * 
 * Para ello usa las estructuras del fichero str_lineas.h comunes con el mdulo
 * de comunicacin generado en el branche \b sock_linux y con el mdulo driver
 * generado en el branche \b driver_linux.
 */
class comunicacion_linux
{
	private:
		int modulo_id;
		int puerto_linux;
		char* ip_linux;
		std::string nombre_fichero;
		configuracion_lineas config_lineas;
		datos_lineas datos;
		ACE_SOCK_Connector conexion;
		ACE_SOCK_Stream socket_aceptado;
		int max_retrasos;
		__u64 turno;
		/** \fn enviar_informacion()
		 * \brief funcin que envia la informacin con la configuracin de las
		 * matrizes de expulsin que sern enviadas para cada imagen tratada.
		 * 
		 * Envia una estructura de tipo configuracion_lineas.
		 */
		void enviar_informacion();
                comando_drv comando_run, comando_stop;
	public:

		/**
		 * \brief Constructor de la clase, obtiene la configuracin para la 
		 * comunicacin con el ordenador de expulsin.
		 * \param fichero apuntador a caracter donde se informa el nombre del fichero de configuracin del mdulo.
		 */
//		comunicacion_linux(const char* fichero, int modulo_id);
		comunicacion_linux(void* fichero, int modulo_id);
		~comunicacion_linux();

		/**
		 * \brief funcin que se conecta al ordenador de expulsin a travs de un
		 * socket.
		 * 
		 * Utiliza los objetos ACE_SOCK_Connector para la conexin.
		 */
		bool conectar();

		/**
		 * \brief funcin que se desconecta del ordenador de expulsin.
		 */
		void desconectar();
		
		/** \fn tratar_matriz(matriz_expulsion* m_expulsion)
		 * \brief Envia la informacin recibida en un objeto de tipo matriz_expulsion
		 * al ordenador de expulsin.
		 * 
		 * Para ello utiliza una estructura de tipo linea_salida.
		 * \param m_expulsion apuntador a una estructura de tipo matriz_expulsion 
		 * en la que hay la informacin de la imagen tratada
		 * \note el tamao de la matriz debe coincidir con la informacin enviada 
		 * al inicio de la comunicacin con el ordenador de expulsin.
		 */
		void tratar_matriz(matriz_expulsion* m_expulsion);

		/** \brief Obtiene los parametros que pueden haber sido modificados por
		 * la interfa del usuario.
		 */
		void obtener_parametros_GUI();
                void configura();
                void run();
                void stop();
                void getset_entero(char* key, int* val, int def);
}; // class comunicacion_linux

}; // namespace picvisa

#endif
