/**
 * \file expulsion.cc
 * \brief Contiene la estructura de funciones necesarias para hacer un mdulo 
 * dring para enviar el resultado de las imagenes tratadas al dispositivo de expulsin.
 * 
 *
*/
#include <iostream>
#include <stdlib.h>
//#include <ace/OS.h>
#include <ace/Event.h>
#include <ace/Auto_Event.h>
#include "dring.h"
#include "comunicacion_linux.h"
#include "modversion.h"
using namespace picvisa;

static comunicacion_linux* c_l;
static bool conectado;

int dr_version(dr_info_mod_t* info) {
    char* version = (char*) info->info;
    int n = info->prioridad;
    snprintf(version, n, "%s", MOD_VERSION);
    return 0;
}

/**
 * \brief No hace nada.
 */
int DLL_EXPORT dr_inicializa(dr_info_mod_t* info_mod)
{
	c_l = new comunicacion_linux((char*)info_mod->info, info_mod->id);
	return 0;
}

/**
 * \brief Crea una instancia de tipo comunicacion_linux que ser la encargada 
 * de enviar el resultado de las imagenes tratadas al ordenador de expulsin.
 */
int DLL_EXPORT dr_marcha(dr_info_mod_t* )
{
  
    return 0;
}
int DLL_EXPORT dr_marcha_secuencial(dr_info_mod_t* )
{
	ACE_DEBUG((LM_INFO, "En dr_marcha_secuencial\n"));
	try
	{
		c_l->obtener_parametros_GUI();
		conectado = c_l->conectar();
		if(conectado){
			c_l->stop();
			c_l->configura();;
			c_l->run();
		}
	}
	catch(comunicacion_excepcion_error& e)
	{
		conectado = false;
	}
	ACE_DEBUG((LM_INFO, "FIN CONEXION\n"));
	return 0;
}

/**
 * \brief no hace nada, los parametros que podemos modificar en este modulo deben
 * modificarse con la aplicacion parada.
 */
int DLL_EXPORT dr_configura(dr_info_mod_t* )
{
//	c_l->obtener_parametros_GUI();
	return 0;
}

/**
 * \brief Recibe el resultado de una imagen tratada y llama a la funcin encargada
 * de enviarla.
 * 
 * Debe recibir un apuntador a una estructura de tipo matriz_expulsion.
 * \return Siempre devuelve 0, porque no debe enviarle el resultado de la operacin
 * a nadie.
 */
int DLL_EXPORT dr_ejecuta(dr_info_mod_t* info_mod)
{
//	ACE_DEBUG((LM_DEBUG, "Ejecuta expulsion (conectado %i)\n", (int)conectado));
	if(conectado)
		c_l->tratar_matriz((matriz_expulsion*)info_mod->info);
	return 0;
}

/**
 * \brief Elimina la instancia del objeto de tipo comunicacion_linux.
 */
int DLL_EXPORT dr_paro(dr_info_mod_t* )
{
    if(conectado) {
        c_l->stop();
        c_l->desconectar();
    }
    return 0;
}

/**
 * \brief Elimina la instancia del objeto de tipo comunicacion_linux.
 */
int DLL_EXPORT dr_aborta(dr_info_mod_t* )
{
//	c_l->desconectar();
	return 0;
}

/**
 * \brief No hace nada.
 */
int DLL_EXPORT dr_finaliza(dr_info_mod_t* )
{
	delete c_l;
	c_l = NULL;
	return 0;
}
