/*
 * \file gpio.c
 *
 * Modulo que gestiona las I/O de la placa LPC1769
 * 
 * Author: Luis Segui
 *
 */
#include "lwip/sys.h"
#include "lwip/api.h"

#include "str_lineas.h"

#include "gpio.h"

/*-----------------------------------------------------------------------------------*/
/*
 * Este mapeo de las valvulas deberia hacer coincidir el numero de valvula con la GPIO concreta.
 */
const int MAPA_VALVULAS[VALVULAS_PUERTO] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23};

/*-----------------------------------------------------------------------------------*/

bool gpio_Set_Outputs_Datos(datos_lineas *dcl_data)
{
	 bool activado = false;

/*
 *  Este bucle lo recorremos byte a byte para mostrar por la UART y facilitar el visualizado con las salidas GPIO
 *  lo normal seria cada bit hacerlo de la estructura datos hacerlo corresponder a una salida y escribir directo los puertos.
*/
	 for(int ii=0;ii<16; ii++){
		 for(int jj=15;jj>=0; jj--) {
			 DEBUGOUT("%02X", dcl_data->lineas[ii].datos[jj]);
			 if ((!activado)&&(dcl_data->lineas[ii].datos[jj])){
				 Chip_GPIO_SetPortOutLow(LPC_GPIO, GPIO0, ~(dcl_data->lineas[ii].datos[jj]));
				 Chip_GPIO_SetPortOutHigh(LPC_GPIO, GPIO0, (dcl_data->lineas[ii].datos[jj]));
				 activado=true;
			 }
		 }
	 }
	DEBUGOUT("\r\n");

	 return activado;
}

/*-----------------------------------------------------------------------------------*/

bool gpio_Set_Outputs_Valor(LPC_GPIO_T *pGPIO, uint8_t portNum, uint32_t bitValue)
{
	Chip_GPIO_SetPortOutLow(LPC_GPIO, GPIO0, 0xFF);

	 return true;
}
/*-----------------------------------------------------------------------------------*/

void gpio_Config_Out(void)
{
	for(int ii=0;ii<VALVULAS_PUERTO; ii++){
		Chip_IOCON_PinMux(LPC_IOCON, GPIO0, MAPA_VALVULAS[ii], IOCON_MODE_PULLDOWN, IOCON_FUNC0);
		Chip_GPIO_SetPinDIROutput(LPC_GPIO, GPIO0, MAPA_VALVULAS[ii]);
	}
}

	/*-----------------------------------------------------------------------------------*/

void gpio_Init(void)
{
	gpio_Config_Out();
	gpio_Set_Outputs_Valor(LPC_GPIO, GPIO0, 0);
}
/*-----------------------------------------------------------------------------------*/
