/**
 * \file str_lineas.h
 * 
 * \brief Contiene las estructuras necesarias para la comunicación con el ordenador
 * de captura y procesado de las imágenes.
 *
 * \author Luis Segui
*/
#ifndef PV_ESTRUCTURA_LINEAS_H
#define PV_ESTRUCTURA_LINEAS_H

#include "tcp_com.h"


// Definicion de los mensajes a recibir de la aplicaci�n

/** \def INFORMACION_CONFIGURACION
 *  \brief Constante que se envia en el primer mensaje para indicar la configuración de la información a enviar a posteriori.
 */
#define ORDEN_CONFIGURACION 'c'
/** \def INFORMACION_LINEAS
 *  \brief Constante que se envia en el cada mensaje con información de una matriz de expulsión.
 */
#define ORDEN_LINEAS 'd'

/** \def ORDEN_RUN
 *  \brief Constante que se envia en el cada mensaje para que empieze a procesar interrupciones.
 */
#define ORDEN_RUN 'r'


/** \def ORDEN_STOP
 *  \brief Constante que se envia en el cada mensaje para que pare de procesar interrupciones.
 */
#define ORDEN_STOP 's'


/** @struct configuracion_lineas
 *  \brief Estructura que sirve para indicar al servidor de expulsion la configuración de la infomación que le enviaremos.
 * 
 * <ul>
 *	<li>\c accion debe ser siempre INFORMACION_CONFIGURACION
 *	<li>\c numero_filas n�mero de filas de la matriz.
 *	<li>\c numero_salidas_driver1 número de columnas pertenecientes al primer driver.
 *	<li>\c numero_salidas_driver2 número de columnas pertenecientes al segundo driver.
 *	<li>\c retardo el tiempo de retardo que debe aplicar el driver.
 * </ul>
*/

typedef struct __configuracion_lineas
{
    u8_t comando;
    u8_t padding;
    u16_t retardo; //retraso a aplicar
    u16_t numero_filas; // numero de filas por paquete
    u16_t max_retrasos; //maximo numero de retrasos permitido
    u16_t divisor;// divisor de interrupciones, sin uso
    u16_t numero_salidas_driver1, numero_salidas_driver2, reserva3, reserva4; //reserva
} configuracion_lineas;

/** @struct linea_salida
 * \brief Estructura que tiene los datos de las imagenes enviada.
 * 
 * <ul>
 *	<li>\c tiempo turno de cada linea
 *      <li>\c salidas Datos 128 bits a escribir en los puertos
 * </ul>
 * 
*/

typedef struct __linea_salida
{
	u8_t datos[16];//128 bits/salidas
} linea_salida;


/** @struct datos_salidas
 * \brief Estructura que sirve de cabecera de las imagenes a enviar.
 *  el limite maximo será de TAM_MAX_LINEA lineas, para no exceder
 *  el tam maximo de un paquete TCP (normalmente 1500 bytes) 
 *  si se quieren enviar mas hay que enviar mas de un paquete
 * <ul>
 *	<li>\c tiempo turno de cada linea
 *      <li>\c salidas Datos 128 bits a escribir en los puertos
 * </ul>
 * 
 */

#define TAM_MAX_TCP (1500-64) //bytes payload ethernet - overhead tcp
#define TAM_MAX_LINEA ((TAM_MAX_TCP - 1)/sizeof(linea_salida))
#pragma pack(push, 1)
typedef struct __datos_lineas
{
	u8_t comando;
	uint64_t tiempo;
    linea_salida lineas[TAM_MAX_LINEA];
} datos_lineas;

typedef struct __datos_driver
{
	uint64_t turno;
	u32_t retrasos_total;
	u32_t retrasos_parcial;
	u32_t desbordes;
} datos_driver;

#pragma pack(pop)
typedef struct __comando_drv {
	u8_t comando ;
} comando_drv;

#endif
