/*
 * Copyright (c) 2001-2003 Swedish Institute of Computer Science.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 */
#include "tcp_com.h"

#include "lwip/opt.h"

#if LWIP_NETCONN

#include "lwip/sys.h"
#include "lwip/api.h"

#include "str_lineas.h"
#include "gpio.h"


/*-----------------------------------------------------------------------------------*/

static void 
tcpecho_thread(void *arg)
{
  struct netconn *conn, *newconn;
  err_t err;
  ip_addr_t ipaddr;
  LWIP_UNUSED_ARG(arg);

  DEBUGOUT("Entramos en tcpecho_thread \r\n");
  /* Create a new connection identifier. */
  conn = netconn_new(NETCONN_TCP);

  /* Bind connection to well known port number 7. */
//  netconn_bind(conn, NULL, 7);
//  IP4_ADDR(&ipaddr, 192, 168, 1, 71);
  err = netconn_bind(conn, NULL, 80);
  if (err != ERR_OK) {
	  DEBUGOUT("Error en BIND\r\n");
  }

  /* Tell connection to go into listening mode. */
  err = netconn_listen(conn);
  if (err != ERR_OK) {
	  DEBUGOUT("Error en LISTEN\r\n");
  }

  DEBUGOUT("Conexion creada, entramos en bucle para recibir \r\n");

  while (1) {

    /* Grab new connection. */
    err = netconn_accept(conn, &newconn);
    DEBUGOUT("accepted new connection %p\r\n", newconn);
    /* Process the new connection. */
    if (err == ERR_OK) {
      struct netbuf *buf;
      void *data;
      configuracion_lineas *config_lineas;
      datos_lineas *dcl_data;
      comando_drv *comando;
      datos_driver resposta_driver;
      u16_t len, port;
      

      while ((err = netconn_recv(newconn, &buf)) == ERR_OK) {
    	  Board_LED_Toggle(0);
    	  err = netconn_getaddr(newconn, &ipaddr, &port, 1);
		  static char tmp_buff[16];
		  DEBUGOUT("Recved. IP_ADDR    : %s\r\n", ipaddr_ntoa_r((const ip_addr_t *) &ipaddr, tmp_buff, 16));
    	  do {
             netbuf_data(buf, &data, &len);
             comando = data;
             switch (comando->comando) {
             	 /* Procesem ordre de configuració per part de la aplicació emissora */
             	 case ORDEN_CONFIGURACION:
             		 config_lineas = data;
             		 DEBUGOUT("DATA RECEIVED. ORDEN CONFIGURACION. Comando: %c, Padding: %i, Retardo: %d, Numero_filas: %d, Max_Retrasos: %d, Divisor: %d, Salidas_driver1: %d, Salidas_driver2: %d \r\n",
            		 config_lineas->comando, ntohs(config_lineas->padding), ntohs(config_lineas->retardo), ntohs(config_lineas->numero_filas), ntohs(config_lineas->max_retrasos), ntohs(config_lineas->numero_salidas_driver1), ntohs(config_lineas->numero_salidas_driver2));
             		 break;
                 /* Procesem ordre de dades per part de la aplicació emissora */
             	 case ORDEN_LINEAS:
             		 dcl_data = data;
             		 DEBUGOUT("DATA RECEIVED. ORDEN LINEAS. Comando: %c, Tiempo: %llu TAM_MAX_LINEA: %d\r\n", dcl_data->comando, dcl_data->tiempo, TAM_MAX_LINEA);
             		 DEBUGOUT("		VALVULAS: ");
             		 bool activado = false;
             		 activado = gpio_Set_Outputs_Datos(dcl_data);

					 /* Preparem resposta a la aplicació emissora */
					 resposta_driver.turno = dcl_data->tiempo;
					 resposta_driver.desbordes = 1;
					 resposta_driver.retrasos_parcial = 2;
					 resposta_driver.retrasos_total = 3;
					 len = sizeof(resposta_driver);

					 DEBUGOUT("Entramos en escritura. Mida= len=%"U16_F" \r\n", len);

					 /* Enviem la resposta */
					 err = netconn_write(newconn, &resposta_driver, len, NETCONN_COPY);
					 if (err == ERR_OK) {
		            	 LWIP_DEBUGF(API_LIB_DEBUG, ("netconn_send_data: sent %p, len=%"U16_F"\r\n", data, len));
		            	 err = netconn_getaddr(newconn, &ipaddr, &port, 0);
		            	 DEBUGOUT("Send. IP_ADDR    : %s\r\n", ipaddr_ntoa_r((const ip_addr_t *) &ipaddr, tmp_buff, 16));
		             }
		             else {
		            	 DEBUGOUT("tcpecho: netconn_write: error \"%s\"\n", lwip_strerr(err));
		             }
		             break;
	             /* Procesem ordre de marxa per part de la aplicació emissora */
             	 case ORDEN_RUN:
             		 DEBUGOUT("DATA RECEIVED. ORDEN RUN. Comando: %c \r\n", comando->comando);
             		 break;
    	         /* Procesem ordre de aturada per part de la aplicació emissora */
             	 case ORDEN_STOP:
             		 DEBUGOUT("DATA RECEIVED. ORDEN STOP. Comando: %c \r\n", comando->comando);
             		 gpio_Set_Outputs_Valor(LPC_GPIO, GPIO0, 0);
             		 break;
             	 default:
             		 DEBUGOUT("DATA RECEIVED. ORDEN DESCONOCIDA.");
             	     break;
             }

#if 0
            if (err != ERR_OK) {
              printf("tcpecho: netconn_write: error \"%s\"\n", lwip_strerr(err));
            }
#endif
        } while (netbuf_next(buf) >= 0);
        netbuf_delete(buf);
      }
	  DEBUGOUT("Got EOF, looping\r\n");
      /* Close connection and discard connection identifier. */
      netconn_close(newconn);
      netconn_delete(newconn);
    }
  }
  DEBUGOUT("FINAL FUNCION\r\n");

}
/*-----------------------------------------------------------------------------------*/
void
tcpecho_init(void)
{
	DEBUGOUT("Creando hilo tcpecho_thread \r\n");
	sys_thread_new("tcpecho_thread", tcpecho_thread, NULL, DEFAULT_THREAD_STACKSIZE, DEFAULT_THREAD_PRIO);
}
/*-----------------------------------------------------------------------------------*/

#endif /* LWIP_NETCONN */
