/**
 * \file str_lineas.h
 * \brief Contiene las estructuras necesarias para la comunicacin con el ordenador
 * de captura y procesado de las imagenes.
 * Este fichero debe mantener-se a la vez que los ficheros con el mismo nombre 
 * en el trunk y el branche sock_linux.
 * 
 *
*/
#ifndef PV_ESTRUCTURA_LINEAS_H
#define PV_ESTRUCTURA_LINEAS_H

#include <linux/types.h>

//typedef unsigned __int64 __u64;
//typedef unsigned __int32 __u32;

// Definicion de los mensajes a recibir de la aplicacin

/** \def INFORMACION_CONFIGURACION
 *  \brief Constante que se envia en el primer mensaje para indicar la configuracin de la informacin a enviar a posteriori.
 */
#define ORDEN_CONFIGURACION 'c'
/** \def INFORMACION_LINEAS
 *  \brief Constante que se envia en el cada mensaje con informacin de una matriz de expulsin.
 */
#define ORDEN_LINEAS 'd'

/** \def ORDEN_RUN
 *  \brief Constante que se envia en el cada mensaje para que empieze a procesar interrupciones.
 */
#define ORDEN_RUN 'r'


/** \def ORDEN_STOP
 *  \brief Constante que se envia en el cada mensaje para que pare de procesar interrupciones.
 */
#define ORDEN_STOP 's'


/** @struct configuracion_lineas
 *  \brief Estructura que sirve para indicar al servidor de expulsion la configuracin de la infomacin que le enviaremos.
 * 
 * <ul>
 *	<li>\c accion debe ser siempre INFORMACION_CONFIGURACION
 *	<li>\c numero_filas nmero de filas de la matriz.
 *	<li>\c numero_salidas_driver1 nmero de columnas pertenecientes al primer driver.
 *	<li>\c numero_salidas_driver2 nmero de columnas pertenecientes al segundo driver.
 *	<li>\c retardo el tiempo de retardo que debe aplicar el driver.
 * </ul>
*/

typedef struct __configuracion_lineas
{
    __u8 comando;
    __u8 padding;
    __u16 retardo; //retraso a aplicar
    __u16 numero_filas; // numero de filas por paquete
    __u16 max_retrasos; //maximo numero de retrasos permitido
    __u16 divisor;// divisor de interrupciones, sin uso
    __u16 reserva1, reserva2, reserva3, reserva4; //reserva
} configuracion_lineas;

/** @struct linea_salida
 * \brief Estructura que tiene los datos de las imagenes enviada.
 * 
 * <ul>
 *	<li>\c tiempo turno de cada linea
 *      <li>\c salidas Datos 128 bits a escribir en los puertos
 * </ul>
 * 
*/

typedef struct __linea_salida
{
    __u8 datos[16];//128 bits/salidas
} linea_salida;


/** @struct datos_salidas
 * \brief Estructura que sirve de cabecera de las imagenes a enviar.
 *  el limite maximo ser de TAM_MAX_LINEA lineas, para no exceder 
 *  el tam maximo de un paquete TCP (normalmente 1500 bytes) 
 *  si se quieren enviar mas hay que enviar mas de un paquete
 * <ul>
 *	<li>\c tiempo turno de cada linea
 *      <li>\c salidas Datos 128 bits a escribir en los puertos
 * </ul>
 * 
 */

#define TAM_MAX_TCP (1500-50) //bytes payload ethernet - overhead tcp
#define TAM_MAX_LINEA ((TAM_MAX_TCP - 1)/sizeof(linea_salida))
#pragma pack(push, 1)

typedef struct __datos_lineas
{
    __u8 comando;
    __u64 tiempo;
    linea_salida lineas[TAM_MAX_LINEA];
} datos_lineas;

typedef struct __datos_driver
{
    __u64 turno;
    __u32 retrasos_total;
    __u32 retrasos_parcial;
    __u32 desbordes;
} datos_driver;
#pragma pack(pop)
typedef struct __comando_drv {
    __u8 comando ;
} comando_drv;

#endif
